@@ -19,10 +19,14 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #
 
+from __future__ import print_function
+
+import six
+
 
 class Dummy(object):
     """Magic "infectious" class.
-    
+
     This class simplifies nice errors on the creation of
     unsupported databases.
     """
@@ -36,4 +40,10 @@ class Dummy(object):
     def __add__(self, other):
         return self
 
+    def __bool__(self):
+        return False
+
+    if six.PY2:
+        __nonzero__ = __bool__
+
 dummy = Dummy()
