@@ -18,13 +18,17 @@
 # You should have received a copy of the GNU Lesser General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #
+from __future__ import print_function
+
 from weakref import ref
 
+import six
+
 from storm.exceptions import ClassInfoError
-from storm.expr import Expr, FromExpr, Column, Desc, TABLE
-from storm.expr import SQLToken, CompileError, compile
+from storm.expr import Column, Desc, TABLE
+from storm.expr import compile, Table
 from storm.event import EventSystem
-from storm import Undef
+from storm import Undef, has_cextensions
 
 
 __all__ = ["get_obj_info", "set_obj_info", "get_cls_info",
@@ -40,9 +44,11 @@ def get_obj_info(obj):
         obj_info = ObjectInfo(obj)
         return obj.__dict__.setdefault("__storm_object_info__", obj_info)
 
+
 def set_obj_info(obj, obj_info):
     obj.__dict__["__storm_object_info__"] = obj_info
 
+
 def get_cls_info(cls):
     if "__storm_class_info__" in cls.__dict__:
         # Can't use attribute access here, otherwise subclassing won't work.
@@ -51,8 +57,9 @@ def get_cls_info(cls):
         cls.__storm_class_info__ = ClassInfo(cls)
         return cls.__storm_class_info__
 
+
 class ClassInfo(dict):
-    """Persistent storm-related information of a class.
+    """Persistent Storm-related information of a class.
 
     The following attributes are defined:
 
@@ -70,8 +77,8 @@ class ClassInfo(dict):
 
         self.cls = cls
 
-        if isinstance(self.table, basestring):
-            self.table = SQLToken(self.table)
+        if isinstance(self.table, six.string_types):
+            self.table = Table(self.table)
 
         pairs = []
         for attr in dir(cls):
@@ -79,7 +86,6 @@ class ClassInfo(dict):
             if isinstance(column, Column):
                 pairs.append((attr, column))
 
-
         pairs.sort()
 
         self.columns = tuple(pair[1] for pair in pairs)
@@ -121,7 +127,6 @@ class ClassInfo(dict):
         self.primary_key_pos = tuple(id_positions[id(column)]
                                      for column in self.primary_key)
 
-
         __order__ = getattr(cls, "__storm_order__", None)
         if __order__ is None:
             self.default_order = Undef
@@ -130,7 +135,7 @@ class ClassInfo(dict):
                 __order__ = (__order__,)
             self.default_order = []
             for item in __order__:
-                if isinstance(item, basestring):
+                if isinstance(item, six.string_types):
                     if item.startswith("-"):
                         prop = Desc(getattr(cls, item[1:]))
                     else:
@@ -145,13 +150,15 @@ class ClassInfo(dict):
     def __ne__(self, other):
         return self is not other
 
+    __hash__ = object.__hash__
+
 
 class ObjectInfo(dict):
 
     __hash__ = object.__hash__
 
     # For get_obj_info(), an ObjectInfo is its own obj_info.
-    __storm_object_info__ = property(lambda self:self)
+    __storm_object_info__ = property(lambda self: self)
 
     def __init__(self, obj):
         # FASTPATH This method is part of the fast path.  Be careful when
@@ -167,9 +174,11 @@ class ObjectInfo(dict):
         self.variables = variables = {}
 
         for column in self.cls_info.columns:
-            variables[column] = column.variable_factory(column=column,
-                                                        event=event)
- 
+            variables[column] = \
+                column.variable_factory(column=column,
+                                        event=event,
+                                        validator_object_factory=self.get_obj)
+
         self.primary_vars = tuple(variables[column]
                                   for column in self.cls_info.primary_key)
 
@@ -180,35 +189,59 @@ class ObjectInfo(dict):
         return self is not other
 
     def set_obj(self, obj):
-        self.get_obj = ref(obj, self._emit_object_deleted)
+        self._ref = ref(obj, self._emit_object_deleted)
+
+    def get_obj(self):
+        return self._ref()
 
     def _emit_object_deleted(self, obj_ref):
         self.event.emit("object-deleted")
 
     def checkpoint(self):
-        for variable in self.variables.itervalues():
+        for variable in six.itervalues(self.variables):
             variable.checkpoint()
 
-try:
+
+if has_cextensions:
     from storm.cextensions import ObjectInfo, get_obj_info
-except ImportError, e:
-    if "cextensions" not in str(e):
-        raise
 
 
-class ClassAlias(FromExpr):
+class ClassAlias(object):
+    """Create a named alias for a Storm class for use in queries.
+
+    This is useful basically when the SQL 'AS' feature is desired in code using
+    Storm queries.
+
+    ClassAliases which are explicitly named (i.e., when 'name' is passed) are
+    cached for as long as the class exists, such that the alias returned from
+    C{ClassAlias(Foo, 'foo_alias')} will be the same object no matter how many
+    times it's called.
+
+    @param cls: The class to create the alias of.
+    @param name: If provided, specify the name of the alias to create.
+    """
 
     alias_count = 0
 
     def __new__(self_cls, cls, name=Undef):
         if name is Undef:
+            use_cache = False
             ClassAlias.alias_count += 1
             name = "_%x" % ClassAlias.alias_count
-        cls_info = get_cls_info(cls)
-        alias_cls = type(cls.__name__+"Alias", (cls, self_cls),
+        else:
+            use_cache = True
+            cache = cls.__dict__.get("_storm_alias_cache")
+            if cache is None:
+                cls._storm_alias_cache = {}
+            elif name in cache:
+                return cache[name]
+        alias_cls = type(cls.__name__ + "Alias", (self_cls,),
                          {"__storm_table__": name})
+        alias_cls.__bases__ = (cls, self_cls)
         alias_cls_info = get_cls_info(alias_cls)
         alias_cls_info.cls = cls
+        if use_cache:
+            cls._storm_alias_cache[name] = alias_cls
         return alias_cls
 
 
